/*
 * @Author: your name
 * @Date: 2021-09-28 12:48:40
 * @LastEditTime: 2022-03-16 00:27:40
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath: /RM/roadmodel/modules/common/road/lane.h
 */
#pragma once
#include <stdint.h>

#include <algorithm>
#include <list>
#include <vector>

#include "enum_list.h"
#include "lane_segment.h"
#include "lane_segment_attribute.h"
#include "macros.h"
namespace rm {

class Lane {
 public:
  void set_lane_id(const uint8_t& lane_id);

  void set_lane_position(const LanePosition& lane_position);

  void set_start_offset(const float& start_offset);

  void set_end_offset(const float& end_offset);

  void set_lane_type_and_speed_limit();

  void add_left_lane_marker_id(const uint32_t& lane_marker_segment_id);

  void add_right_lane_marker_id(const uint32_t& lane_marker_segment_id);

  LaneSegment* add_front_lane_segment();
  LaneSegment* add_back_lane_segment();

  LaneSegmentAttribute* add_lane_segment_attribute();

  void set_is_start_out_of_range(const bool& is_start_out_of_range);

  void set_is_end_out_of_range(const bool& is_end_out_of_range);

  void set_is_left_lane_marker_changed(const bool is_left_lane_marker_changed);

  void set_is_right_lane_marker_changed(
      const bool is_right_lane_marker_changed);

  uint8_t lane_id() const;

  LanePosition lane_position() const;

  float start_offset() const;

  float end_offset() const;

  const std::list<uint32_t>& left_lane_marker_id() const;

  const std::list<uint32_t>& right_lane_marker_id() const;

  std::list<uint32_t>* mutable_left_lane_marker_id();

  std::list<uint32_t>* mutable_right_lane_marker_id();

  const std::vector<LaneSegment>& front_lane_segment() const {
    return front_lane_segment_;
  }

  std::vector<LaneSegment>* mutable_front_lane_segment() {
    return &front_lane_segment_;
  }

  const std::vector<LaneSegment>& back_lane_segment() const {
    return back_lane_segment_;
  }

  std::vector<LaneSegment>* mutable_back_lane_segment() {
    return &back_lane_segment_;
  }

  const std::vector<LaneSegmentAttribute>& lane_segment_attribute() const {
    return lane_segment_attribute_;
  }

  std::vector<LaneSegmentAttribute>* mutable_lane_segment_attribute() {
    return &lane_segment_attribute_;
  }

  bool is_start_out_of_range() const { return is_start_out_of_range_; }

  bool is_end_out_of_range() const { return is_end_out_of_range_; }

  bool is_left_lane_marker_changed() const {
    return is_left_lane_marker_changed_;
  }

  bool is_right_lane_marker_changed() const {
    return is_right_lane_marker_changed_;
  }

  bool is_lane_markers_changed() const {
    return is_left_lane_marker_changed_ || is_right_lane_marker_changed_;
  }

  const std::pair<float, float>& speed_limit() const {
    return speed_limit_;
  }

  /**
   * @brief: 友元函数打印该类数据
   * @param {*}
   * @return {*}
   */
  friend std::ostream& operator<<(std::ostream& os, const Lane& lane);

  rm::LaneType lane_type() const { return lane_type_; }

  // return the lane width of the first lane segment in lane segment list
  float lane_width() const;

  float lane_merge_distance() const;

 private:
  uint8_t lane_id_;
  LaneType lane_type_;
  LaneDirection lane_direction_;
  LanePosition lane_position_;
  std::vector<LaneSegment> front_lane_segment_;
  std::vector<LaneSegment> back_lane_segment_;
  std::list<uint32_t> left_lane_marker_id_;
  std::list<uint32_t> right_lane_marker_id_;
  float start_offset_;
  float end_offset_;
  bool is_start_out_of_range_;
  bool is_end_out_of_range_;
  std::vector<LaneSegmentAttribute> lane_segment_attribute_;
  bool is_left_lane_marker_changed_;
  bool is_right_lane_marker_changed_;
  std::pair<float, float> speed_limit_;
};
}  // namespace rm